package com.dzeio.openhealth.core

import androidx.lifecycle.LiveData
import androidx.lifecycle.MutableLiveData
import kotlinx.coroutines.channels.Channel
import kotlinx.coroutines.flow.callbackFlow

/**
 * Simple Observable implementation
 */
open class Observable<T>(baseValue: T) {

    private val functionObservers: ArrayList<(T) -> Unit> = ArrayList()

    fun addObserver(fn: (T) -> Unit) {
        if (!functionObservers.contains(fn)) {
            functionObservers.add(fn)
        }
    }

    fun addOneTimeObserver(fn: (T) -> Unit) {
        val index = functionObservers.size
        functionObservers.add {
            fn(it)
            functionObservers.removeAt(index)
        }
    }

    fun removeObserver(fn: (T) -> Unit) {
        if (functionObservers.contains(fn)) {
            functionObservers.remove(fn)
        }
    }

    open var value: T = baseValue
        set(value) {
            field = value
            notifyObservers()
        }

    fun notifyObservers() {
        // Notify Functions
        for (fn in functionObservers) {
            notifyObserver(fn)
        }
    }

    fun notifyObserver(observer: (T) -> Unit) {
        observer.invoke(value)
    }

    fun toLiveData(): LiveData<T> {
        val ld = MutableLiveData(value)
        addObserver {
            ld.postValue(it)
        }
        return ld
    }

    /**
     * Transform the observable to a Kotlin Channel
     */
    fun toChannel(): Channel<T> = Channel<T>(Channel.RENDEZVOUS).apply {
        addObserver {
            trySend(it)
        }
    }

    fun toFlow() = callbackFlow {
        addObserver {
            trySend(it)
        }
    }
}
