package com.dzeio.openhealth.data.weight

import androidx.room.ColumnInfo
import androidx.room.Entity
import androidx.room.PrimaryKey
import com.dzeio.openhealth.units.ActivityLevel
import java.sql.Date
import java.text.DateFormat.getDateInstance
import kotlin.math.roundToInt

@Entity()
data class Weight(
    @PrimaryKey(autoGenerate = true)
    var id: Long = 0,
    /**
     * Store the weight in kilograms
     */
    var weight: Float = 0f,

    /**
     * when the weight was taken precise to the millisecond
     */
    @ColumnInfo(index = true)
    var timestamp: Long = System.currentTimeMillis(),

    /**
     * the source for the Entry
     *
     * note: Unused currently but kept for future usage
     */
    var source: String = "OpenHealth",

    /**
     * The BMI
     *
     * calculated from the height and Weight of the user
     *
     * ex: weight / (height(cm)/100)²
     */
    var bmi: Float? = null,

    /**
     * the total body water (tbw) in percents
     *
     * can be estimated
     * https://www.mdapp.co/total-body-water-tbw-calculator-448/
     */
    var totalBodyWater: Float? = null,

    /**
     * the Muscle weight in percents
     */
    var muscles: Float? = null,

    /**
     * the lean Body Mass in percents
     */
    var leanBodyMass: Float? = null,

    /**
     * the Body Fat in percents
     */
    var bodyFat: Float? = null,

    /**
     * the bone Mass in Percents
     */
    var boneMass: Float? = null,

    /**
     * visceral fat in it's own unit?
     */
    var visceralFat: Float? = null,

    /**
     * the BMR rate
     */
    var basalMetabolicRate: Int? = null,

    var totalDailyEnergyExpendure: Int? = null
) {

    /**
     * Run estimations to calculate & set different elements if they are not set
     *
     * @param height the height in `cm`
     * @param age the user's age
     * @param biologicalSex the user's biological sex (female = 0, male = 1)
     * @param activityLevel the level of activity of the user
     */
    fun setItemsWithEstimation(
        height: Int? = null,
        age: Int? = null,
        biologicalSex: Int? = null,
        activityLevel: ActivityLevel? = null
    ) {
        if (bmi == null && height != null) {
            val tmpHeight = (height / 100f)
            bmi = weight / (tmpHeight * tmpHeight)
        }

        if (totalBodyWater == null && height != null) {
            // female = 0, male = 1
            totalBodyWater = (
                if (biologicalSex == 0) {
                    ((0.34454 * height) + (0.183809 * weight) - 35.270121)
                } else {
                    ((0.194786 * height) + (0.296785 * weight) - 14.012934)
                }
                ).toFloat()
        }
        if (basalMetabolicRate == null && height != null && age != null) {
            basalMetabolicRate = (
                if (biologicalSex == 0) {
                    10 * weight + 6.25 * height - 5 * age - 161
                } else {
                    10 * weight + 6.25 * height - 5 * age + 5
                }
                ).toInt()
        }
        if (totalDailyEnergyExpendure == null && activityLevel != null && basalMetabolicRate != null) {
            totalDailyEnergyExpendure = (basalMetabolicRate!! * activityLevel.modifier).roundToInt()
        }
    }

    fun formatTimestamp(): String = getDateInstance().format(Date(timestamp))

    override fun equals(other: Any?): Boolean {
        if (other !is Weight) {
            return super.equals(other)
        }

        return weight == other.weight &&
            timestamp == other.timestamp &&
            bmi == other.bmi &&
            totalBodyWater == other.totalBodyWater &&
            muscles == other.muscles &&
            leanBodyMass == other.leanBodyMass &&
            bodyFat == other.bodyFat &&
            boneMass == other.boneMass &&
            visceralFat == other.visceralFat
    }

    override fun hashCode(): Int {
        var result = id.hashCode()
        result = 31 * result + weight.hashCode()
        result = 31 * result + timestamp.hashCode()
        result = 31 * result + source.hashCode()
        result = 31 * result + (bmi?.hashCode() ?: 0)
        result = 31 * result + (totalBodyWater?.hashCode() ?: 0)
        result = 31 * result + (muscles?.hashCode() ?: 0)
        result = 31 * result + (leanBodyMass?.hashCode() ?: 0)
        result = 31 * result + (bodyFat?.hashCode() ?: 0)
        result = 31 * result + (boneMass?.hashCode() ?: 0)
        result = 31 * result + (visceralFat?.hashCode() ?: 0)
        return result
    }
}
