package com.dzeio.openhealth.devices

import android.Manifest
import android.annotation.SuppressLint
import android.bluetooth.BluetoothDevice
import android.bluetooth.BluetoothGatt
import android.bluetooth.BluetoothGattCharacteristic
import android.bluetooth.BluetoothGattDescriptor
import android.util.Log
import androidx.annotation.RequiresPermission
import com.dzeio.openhealth.core.Observable
import com.dzeio.openhealth.data.weight.Weight
import com.dzeio.openhealth.devices.libs.MiScaleLib
import com.dzeio.openhealth.utils.Bluetooth
import com.dzeio.openhealth.utils.Configuration
import java.text.SimpleDateFormat
import java.util.Calendar
import java.util.Random
import java.util.UUID

class DeviceMiSmartScale2(
    bluetooth: Bluetooth,
    config: Configuration
) : BluetoothLeGattDevice(bluetooth, config) {

    companion object {
        val TAG = this::class.java.name
    }

    override val name = "Mi Smart Scale 2"

    private val WEIGHT_MEASUREMENT_HISTORY_CHARACTERISTIC =
        UUID.fromString("00002a2f-0000-3512-2118-0009af100700")
    private val WEIGHT_CUSTOM_SERVICE = UUID.fromString("00001530-0000-3512-2118-0009af100700")
    private val WEIGHT_CUSTOM_CONFIG = UUID.fromString("00001542-0000-3512-2118-0009af100700")

    private val CONFIG_USER_ID = "com.dzeio.open-health.devices.mi-scale-2.id"

    private val fetchStatus = Observable(FetchStatus(0, 5))

    @SuppressLint("MissingPermission")
    override fun isOfType(item: BluetoothDevice): Boolean =
        item.name == "MIBFS"

    @RequiresPermission(Manifest.permission.BLUETOOTH_CONNECT)
    override fun fetchWeights(): Observable<FetchStatus> {
        fetchStatus.value = FetchStatus(0, 5)
        // step 0
        writeBytes(
            WEIGHT_CUSTOM_SERVICE,
            WEIGHT_CUSTOM_CONFIG,
            byteArrayOf(
                0x06,
                0x04,
                0x00,
                0x00
            )
        )

        return fetchStatus
    }

    @RequiresPermission(Manifest.permission.BLUETOOTH_CONNECT)
    override fun onCharacteristicWrite(
        gatt: BluetoothGatt?,
        characteristic: BluetoothGattCharacteristic?,
        status: Int
    ) {
        if (characteristic?.uuid == WEIGHT_CUSTOM_CONFIG) {
            // step 1
            // set current time
            val currentDateTime: Calendar = Calendar.getInstance()
            val year: Int = currentDateTime.get(Calendar.YEAR)
            val month = (currentDateTime.get(Calendar.MONTH) + 1)
            val day = currentDateTime.get(Calendar.DAY_OF_MONTH)
            val hour = currentDateTime.get(Calendar.HOUR_OF_DAY)
            val min = currentDateTime.get(Calendar.MINUTE)
            val sec = currentDateTime.get(Calendar.SECOND)

            val dateTimeByte = byteArrayOf(
                year.toByte(),
                (year shr 8).toByte(),
                month.toByte(),
                day.toByte(),
                hour.toByte(),
                min.toByte(),
                sec.toByte(),
                0x03,
                0x00,
                0x00
            )

            writeBytes(
                BluetoothLeGattUuid.SERVICE_BODY_COMPOSITION,
                BluetoothLeGattUuid.CHARACTERISTIC_CURRENT_TIME,
                dateTimeByte
            )
            fetchStatus.value.progress++
            fetchStatus.notifyObservers()
        } else if (characteristic?.uuid == BluetoothLeGattUuid.CHARACTERISTIC_CURRENT_TIME) {
            // step 2
            setNotification(
                BluetoothLeGattUuid.SERVICE_BODY_COMPOSITION,
                WEIGHT_MEASUREMENT_HISTORY_CHARACTERISTIC,
                true
            )
            fetchStatus.value.progress++
            fetchStatus.notifyObservers()
        } else if (
            fetchStatus.value.progress < 4 &&
            characteristic?.uuid == WEIGHT_MEASUREMENT_HISTORY_CHARACTERISTIC
        ) {
            // step 4
            writeBytes(
                BluetoothLeGattUuid.SERVICE_BODY_COMPOSITION,
                WEIGHT_MEASUREMENT_HISTORY_CHARACTERISTIC,
                byteArrayOf(
                    0x02
                )
            )
            fetchStatus.value.progress++
            fetchStatus.notifyObservers()
        }
    }

    @RequiresPermission(Manifest.permission.BLUETOOTH_CONNECT)
    override fun onDescriptorWrite(
        gatt: BluetoothGatt?,
        descriptor: BluetoothGattDescriptor?,
        status: Int
    ) {
        // step 3
        super.onDescriptorWrite(gatt, descriptor, status)
        val id = getID()
        writeBytes(
            BluetoothLeGattUuid.SERVICE_BODY_COMPOSITION,
            WEIGHT_MEASUREMENT_HISTORY_CHARACTERISTIC,
            byteArrayOf(
                0x01.toByte(),
                0xFF.toByte(),
                0xFF.toByte(),
                (id and 0xFF00 shl 8).toByte(),
                (id and 0x00FF shl 0).toByte()
            )
        )
        fetchStatus.value.progress++
        fetchStatus.notifyObservers()
    }

    @RequiresPermission(Manifest.permission.BLUETOOTH_CONNECT)
    override fun onCharacteristicChanged(
        gatt: BluetoothGatt,
        characteristic: BluetoothGattCharacteristic,
        value: ByteArray
    ) {
        super.onCharacteristicChanged(gatt, characteristic, value)

        if (value.isNotEmpty()) {
            // step 5+x
            if (value[0] == 0x03.toByte()) {
                Log.d(TAG, "Stop signal received")
                writeBytes(
                    BluetoothLeGattUuid.SERVICE_BODY_COMPOSITION,
                    WEIGHT_MEASUREMENT_HISTORY_CHARACTERISTIC,
                    byteArrayOf(0x03)
                )

                val id = getID()

                val userIdentifier = byteArrayOf(
                    0x04.toByte(),
                    0xFF.toByte(),
                    0xFF.toByte(),
                    (id and 0xFF00 shr 8).toByte(),
                    (id and 0xFF shr 0).toByte()
                )
                writeBytes(
                    BluetoothLeGattUuid.SERVICE_BODY_COMPOSITION,
                    WEIGHT_MEASUREMENT_HISTORY_CHARACTERISTIC,
                    userIdentifier
                )
                fetchStatus.value.progress++
                fetchStatus.notifyObservers()
            } else if (value.size == 13) {
                // 4+x
                Log.d(TAG, "Measurement received")
                val weight = decodeEntry(value)
                if (weight == null) {
                    return
                }

                fetchStatus.value.data.add(weight)
                fetchStatus.value.progress++
                fetchStatus.value.progressMax++
                fetchStatus.notifyObservers()
            }
        }
    }

    override fun reset() {
        config.getInt(CONFIG_USER_ID).value = null
    }

    private fun getID(): Int {
        val id = config.getInt(CONFIG_USER_ID)
        if (id.value == null) {
            id.value = (Random().nextInt(65535 - 100 + 1) + 100)
        }
        return id.value!! + 1
    }

    /**
     * Decode the 13 bytes entry into a [Weight] object
     *
     * control bytes
     * XX XX 00 00 00 00 00 00 00 00 00 00 00
     *
     * Datetime bytes
     * 00 00 XX XX XX XX XX XX XX 00 00 00 00
     *
     * Impedance Bytes
     * 00 00 00 00 00 00 00 00 00 XX XX 00 00
     *
     * Weight Bytes
     * 00 00 00 00 00 00 00 00 00 00 00 XX XX
     */
    @SuppressLint("SimpleDateFormat")
    private fun decodeEntry(entry: ByteArray): Weight? {
        // byte 0
        val ctrlByte0 = entry[0]
        // byte 1
        val ctrlByte1 = entry[1]

        val isLBSUnit = ctrlByte0.isBitSet(0)
        val isImpedance = ctrlByte1.isBitSet(1)
        val isStablilized = ctrlByte1.isBitSet(5)
        val isCattyUnit = ctrlByte1.isBitSet(6)
        val isWeightRemoved = ctrlByte1.isBitSet(7)

        if (isWeightRemoved || !isStablilized) {
            return null
        }

        // byte 2 to 8 represent the datetime
        val year = ((entry[3].toInt() and 0xFF) shl 8) or (entry[2].toInt() and 0xFF)
        val month = entry[4].toInt()
        val day = entry[5].toInt()
        val hours = entry[6].toInt()
        val min = entry[7].toInt()
        val sec = entry[8].toInt()

        val weightTmp = ((entry[12].toInt() and 0xFF) shl 8) or (entry[11].toInt() and 0xFF)
        val weightFloat = if (isLBSUnit || isCattyUnit) weightTmp / 100f else weightTmp / 200f

        val weight = Weight(
            weight = weightFloat
        )

        var impedance: Float? = null
        if (isImpedance) {
            impedance =
                (((entry[10].toInt() and 0xFF) shl 8) or (entry[9].toInt() and 0xFF)).toFloat()
        }

        val dateStr = "$year/$month/$day/$hours/$min/$sec"
        val date = SimpleDateFormat("yyyy/MM/dd/HH/mm/ss").parse(dateStr)?.time

        if (date == null) {
            return null
        }

        weight.timestamp = date

        if (impedance == null) {
            return weight
        }

        val miScaleLib = MiScaleLib(1, 24, 179f)

        weight.apply {
            bmi = miScaleLib.getBMI(weightFloat)
            totalBodyWater = miScaleLib.getWater(weightFloat, impedance)
            visceralFat = miScaleLib.getVisceralFat(weightFloat)
            bodyFat = miScaleLib.getBodyFat(weightFloat, impedance)
            muscles = miScaleLib.getMuscle(weightFloat, impedance) / weightFloat * 100
            leanBodyMass = miScaleLib.getLBM(weightFloat, impedance) / weightFloat * 100
            boneMass = miScaleLib.getBoneMass(weightFloat, impedance)
        }

        return weight
    }

    private fun Byte.isBitSet(pos: Int): Boolean {
        val nn = this.toInt() shr pos
        return (nn and 1) == 1
    }
}
