package com.dzeio.openhealth.ui

import android.app.NotificationChannel
import android.app.NotificationManager
import android.content.Context
import android.content.res.Configuration
import android.os.Build
import android.os.Bundle
import android.util.Log
import android.view.LayoutInflater
import android.view.Menu
import android.view.MenuItem
import android.view.WindowInsets
import android.view.WindowManager
import androidx.core.view.WindowCompat
import androidx.core.view.updatePadding
import androidx.navigation.NavController
import androidx.navigation.fragment.NavHostFragment
import androidx.navigation.ui.AppBarConfiguration
import androidx.navigation.ui.NavigationUI
import androidx.navigation.ui.navigateUp
import androidx.navigation.ui.setupActionBarWithNavController
import androidx.navigation.ui.setupWithNavController
import com.dzeio.openhealth.R
import com.dzeio.openhealth.core.BaseActivity
import com.dzeio.openhealth.databinding.ActivityMainBinding
import com.dzeio.openhealth.interfaces.NotificationChannels
import com.dzeio.openhealth.services.OpenHealthService
import com.dzeio.openhealth.utils.ServiceUtils
import com.dzeio.openhealth.workers.WaterReminderWorker
import dagger.hilt.android.AndroidEntryPoint

@AndroidEntryPoint
class MainActivity : BaseActivity<ActivityMainBinding>() {

    companion object {
        val TAG: String = this::class.java.simpleName
    }

    private lateinit var appBarConfiguration: AppBarConfiguration

    private lateinit var navController: NavController

    override val bindingInflater: (LayoutInflater) -> ActivityMainBinding =
        ActivityMainBinding::inflate

    override fun onCreate(savedInstanceState: Bundle?) {
        setTheme(R.style.Theme_OpenHealth_NoActionBar)
        super.onCreate(savedInstanceState)
    }

    override fun onCreated(savedInstanceState: Bundle?) {
        super.onCreated(savedInstanceState)

        setSupportActionBar(binding.toolbar)

        // Comportement chelou API 28-
        // Comportement normal 31+
        // do not do the cool status/navigation bars for API 29 & 30
        if (Build.VERSION.SDK_INT != Build.VERSION_CODES.R && Build.VERSION.SDK_INT != Build.VERSION_CODES.Q) {
            // allow to put the content behind the status bar & Navigation bar (one of them at least lul)
            // ALSO: make the status/navigation bars semi-transparent
            window.addFlags(WindowManager.LayoutParams.FLAG_TRANSLUCENT_NAVIGATION)
            window.addFlags(WindowManager.LayoutParams.FLAG_TRANSLUCENT_STATUS)

            // Update toolbar height with the statusbar size included
            // ALSO: make both the status/navigation bars transparent (WHYYYYYYY)
            val toolbarHeight = binding.toolbar.layoutParams.height
            window.decorView.setOnApplyWindowInsetsListener { _, insets ->
                // Use getInsets(int) with WindowInsets.Type.systemBars() instead.
                val statusBarSize = if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.R) {
                    insets.getInsets(WindowInsets.Type.systemBars()).top
                } else {
                    insets.systemWindowInsetTop
                }
                // Add padding to the toolbar (YaY I know how something works)
                binding.toolbar.updatePadding(top = statusBarSize)
                binding.toolbar.layoutParams.height = toolbarHeight + statusBarSize
                return@setOnApplyWindowInsetsListener insets
            }

            // normally makes sure icons are at the correct color
            when (this.resources.configuration.uiMode.and(Configuration.UI_MODE_NIGHT_MASK)) {
                Configuration.UI_MODE_NIGHT_YES -> {
                    WindowCompat.getInsetsController(window, window.decorView).apply {
                        // force to display the bars in light color
                        isAppearanceLightNavigationBars = true
                        isAppearanceLightStatusBars = false // WHY
                    }
                }
                Configuration.UI_MODE_NIGHT_NO, Configuration.UI_MODE_NIGHT_UNDEFINED -> {
                    WindowCompat.getInsetsController(window, window.decorView).apply {
                        // force to display the bars in dark color
                        isAppearanceLightNavigationBars = false
                        isAppearanceLightStatusBars = true // WHY
                    }
                }
            }
        }

        val navHostFragment =
            supportFragmentManager.findFragmentById(R.id.nav_host_fragment) as NavHostFragment

        navController = navHostFragment.navController

        appBarConfiguration = AppBarConfiguration(
            setOf(
                R.id.nav_home,
                R.id.nav_browse,
                R.id.nav_activity,
                R.id.nav_extensions
            )
        )

        setupActionBarWithNavController(navController, appBarConfiguration)

        binding.bottomNav.setupWithNavController(navController)

        createNotificationChannel()

        // Services
        WaterReminderWorker.setup(this)

        ServiceUtils.startService(this, OpenHealthService::class.java)
    }

    override fun onCreateOptionsMenu(menu: Menu): Boolean {
        menuInflater.inflate(R.menu.main, menu)
        return super.onCreateOptionsMenu(menu)
    }

    override fun onSupportNavigateUp(): Boolean =
        navController.navigateUp(appBarConfiguration) || super.onSupportNavigateUp()

    override fun onOptionsItemSelected(item: MenuItem): Boolean =
        NavigationUI.onNavDestinationSelected(item, navController) ||
            super.onOptionsItemSelected(item)

    private fun createNotificationChannel() {
        val notificationManager: NotificationManager =
            getSystemService(Context.NOTIFICATION_SERVICE) as NotificationManager

        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.O) {
            for (channel in NotificationChannels.values()) {
                Log.d(TAG, channel.channelName)
                try {
                    notificationManager.createNotificationChannel(
                        NotificationChannel(
                            channel.id,
                            channel.channelName,
                            channel.importance
                        )
                    )
                } catch (e: Exception) {
                    Log.e(TAG, "Error Creating Notification Channel", e)
                }
            }
        }
    }
}
