package com.dzeio.openhealth.ui.home

import androidx.lifecycle.LiveData
import androidx.lifecycle.MutableLiveData
import androidx.lifecycle.viewModelScope
import com.dzeio.openhealth.Settings
import com.dzeio.openhealth.core.BaseViewModel
import com.dzeio.openhealth.data.step.StepRepository
import com.dzeio.openhealth.data.water.Water
import com.dzeio.openhealth.data.water.WaterRepository
import com.dzeio.openhealth.data.weight.Weight
import com.dzeio.openhealth.data.weight.WeightRepository
import com.dzeio.openhealth.units.Mass
import com.dzeio.openhealth.units.Volume
import com.dzeio.openhealth.utils.Configuration
import dagger.hilt.android.lifecycle.HiltViewModel
import javax.inject.Inject
import kotlinx.coroutines.flow.collectLatest
import kotlinx.coroutines.launch

@HiltViewModel
class HomeViewModel @Inject internal constructor(
    private val weightRepository: WeightRepository,
    private val waterRepository: WaterRepository,
    stepRepository: StepRepository,
    config: Configuration
) : BaseViewModel() {

    private val _steps = MutableLiveData(0)

    /**
     * Steps taken today by the user
     */
    val steps: LiveData<Int> = _steps

    /**
     * Number of steps the use should do today
     */
    val stepsGoal: LiveData<Int?> = config.getInt(Settings.STEPS_GOAL).toLiveData()

    private val _water = MutableLiveData<Water?>(null)

    /**
     * Quantity of water the user drank today
     */
    val water: LiveData<Water?> = _water

    private val _weights = MutableLiveData<List<Weight>?>(null)

    /**
     * The list of weight of the user
     */
    val weights: LiveData<List<Weight>?> = _weights

    /**
     * The size of a cup for the quick water intake add
     */
    var waterCupSize = config.getInt(Settings.WATER_INTAKE_SIZE).toLiveData()

    private val _waterUnit = MutableLiveData(Volume.MILLILITER)
    val waterUnit: LiveData<Volume> = _waterUnit

    private val _massUnit = MutableLiveData(Mass.KILOGRAM)

    /**
     * The Mass unit used by the user
     */
    val massUnit: LiveData<Mass> = _massUnit

    /**
     * the User weight goal
     */
    val goalWeight = config.getFloat(Settings.WEIGHT_GOAL).toLiveData()

    private val _dailyWaterIntake = MutableLiveData<Int?>(null)
    val dailyWaterIntake: LiveData<Int?> = _dailyWaterIntake

    init {
        // Fetch today's water intake
        viewModelScope.launch {
            waterRepository.todayWater().collectLatest {
                _water.postValue(it)
            }
        }

        // Fetch the user weights
        viewModelScope.launch {
            _steps.postValue(stepRepository.todaySteps())
        }

        // fetch the user weights
        viewModelScope.launch {
            weightRepository.getWeights().collectLatest {
                _weights.postValue(it)
            }
        }

        config.getInt(Settings.MASS_UNIT).apply {
            addObserver {
                if (it == null) return@addObserver
                _massUnit.postValue(Mass.find(it))
            }
            if (value != null) {
                _massUnit.postValue(Mass.find(value!!))
            }
        }

        config.getInt((Settings.VOLUME_UNIT)).apply {
            addObserver {
                if (it === null) return@addObserver
                _waterUnit.postValue(Volume.find(it))
            }
            if (value != null) _waterUnit.postValue(Volume.find(value!!))
        }

        config.getInt(Settings.WATER_INTAKE_DAILY_GOAL).apply {
            addObserver {
                if (it === null) return@addObserver
                _dailyWaterIntake.postValue((it * (_waterUnit.value?.modifier ?: 0f)).toInt())
            }
            if (value != null) _dailyWaterIntake.postValue((value!! * (_waterUnit.value?.modifier ?: 0f)).toInt())
        }
    }

    fun updateWater(water: Water) {
        viewModelScope.launch {
            waterRepository.addWater(water)
        }
    }

    fun deleteWater(item: Water) {
        viewModelScope.launch {
            waterRepository.deleteWater(item)
            _water.postValue(null)
        }
    }
}
