package com.dzeio.openhealth.ui.steps

import android.graphics.Paint
import android.os.Bundle
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import androidx.navigation.fragment.findNavController
import androidx.navigation.fragment.navArgs
import androidx.recyclerview.widget.LinearLayoutManager
import com.dzeio.charts.Entry
import com.dzeio.charts.axis.Line
import com.dzeio.charts.series.BarSerie
import com.dzeio.openhealth.Application
import com.dzeio.openhealth.R
import com.dzeio.openhealth.adapters.ItemAdapter
import com.dzeio.openhealth.core.BaseFragment
import com.dzeio.openhealth.data.step.Step
import com.dzeio.openhealth.databinding.FragmentStepsHomeBinding
import com.dzeio.openhealth.utils.ChartUtils
import dagger.hilt.android.AndroidEntryPoint
import java.text.DateFormat
import java.util.Calendar
import java.util.Date
import java.util.Locale
import java.util.TimeZone
import kotlin.math.roundToInt

@AndroidEntryPoint
class StepsHomeFragment :
    BaseFragment<StepsHomeViewModel, FragmentStepsHomeBinding>(StepsHomeViewModel::class.java) {

    companion object {
        const val TAG = "${Application.TAG}/SHFragment"
    }

    private val args: StepsHomeFragmentArgs by navArgs()

    override val bindingInflater: (LayoutInflater, ViewGroup?, Boolean) -> FragmentStepsHomeBinding =
        FragmentStepsHomeBinding::inflate

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)

        viewModel.init()

        val isDay = args.day > 0L

        val recycler = binding.list

        val manager = LinearLayoutManager(requireContext())
        recycler.layoutManager = manager

        val adapter = ItemAdapter<Step>()

        if (!isDay) {
            adapter.onItemClick = {
                findNavController().navigate(
                    StepsHomeFragmentDirections.actionNavStepsHomeSelf().apply {
                        day = it.value.timestamp
                        title = "Steps from " + it.value.formatTimestamp(true)
                    }
                )
            }
        }
        recycler.adapter = adapter

        val chart = binding.chart

        // setup serie
        val serie = BarSerie(chart)

        chart.apply {
            ChartUtils.materielTheme(chart, requireView())
            yAxis.apply {
                setYMin(0f)
            }

            xAxis.apply {
                dataWidth = if (isDay) 8.64e+7 else 6.048e+8
                scrollEnabled = !isDay
                textPaint.textSize = 32f
                onValueFormat = onValueFormat@{
                    val formatter = if (isDay) {
                        DateFormat.getTimeInstance(
                            DateFormat.SHORT,
                            Locale.getDefault()
                        )
                    } else {
                        DateFormat.getDateInstance(
                            DateFormat.SHORT,
                            Locale.getDefault()
                        )
                    }
                    return@onValueFormat formatter.format(Date(it.toLong()))
                }
            }
            annotator.annotationTitleFormat = { "${it.y.roundToInt()} steps" }
            annotator.annotationSubTitleFormat = annotationSubTitleFormat@{
                val formatter = if (isDay) {
                    DateFormat.getTimeInstance(
                        DateFormat.SHORT,
                        Locale.getDefault()
                    )
                } else {
                    DateFormat.getDateInstance(
                        DateFormat.SHORT,
                        Locale.getDefault()
                    )
                }
                return@annotationSubTitleFormat formatter.format(Date(it.x.toLong()))
            }
        }

        viewModel.goal.observe(viewLifecycleOwner) {
            if (it != null && !isDay) {
                chart.yAxis.addLine(
                    it.toFloat(),
                    Line(true, Paint(chart.yAxis.linePaint).apply { strokeWidth = 4f })
                )
                chart.refresh()
            }
        }

        viewModel.items.observe(viewLifecycleOwner) { list ->
            if (list.isEmpty()) {
                adapter.set(arrayListOf())
                return@observe
            }

            val filtered = if (!isDay) {
                list
            } else {
                list.filter {
                    it.getDay() == args.day
                }
            }
            if (isDay) {
                adapter.set(
                    filtered.map {
                        ItemAdapter.Item(
                            it,
                            getString(
                                R.string.steps_count,
                                it.value
                            ),
                            it.formatTimestamp(false)
                        )
                    }
                )
                serie.entries = filtered.map {
                    Entry(
                        it.timestamp.toDouble(),
                        it.value.toFloat()
                    )
                } as ArrayList<Entry>
            } else {
                val entries: HashMap<Long, Entry> = HashMap()

                list.forEach {
                    val cal = Calendar.getInstance(TimeZone.getTimeZone("UTC"))
                    cal.timeInMillis = it.timestamp

                    cal.set(Calendar.HOUR, 0)
                    cal.set(Calendar.AM_PM, Calendar.AM)
                    val ts = cal.timeInMillis
                    if (!entries.containsKey(ts)) {
                        entries[ts] = Entry((ts).toDouble(), 0F, chart.yAxis.goalLinePaint.color)
                    }

                    entries[ts]!!.y += it.value.toFloat()

                    if (viewModel.goal.value != null) {
                        if (entries[ts]!!.y > viewModel.goal.value!!) {
                            entries[ts]!!.color = null
                        }
                    } else {
                        entries[ts]!!.color = null
                    }
                }

                adapter.set(
                    entries.map { Step(value = it.value.y.toInt(), timestamp = it.key) }
                        .sortedByDescending { it.timestamp }
                        .map {
                            ItemAdapter.Item(
                                it,
                                getString(
                                    R.string.steps_count,
                                    it.value
                                ),
                                it.formatTimestamp(true),
                                icon = R.drawable.ic_zoom_out_map
                            )
                        }
                )

                serie.entries = ArrayList(entries.values)
            }

            chart.xAxis.x =
                chart.xAxis.getXMax() - chart.xAxis.dataWidth!! + chart.xAxis.dataWidth!! / (if (isDay) 24 else 7)

            chart.refresh()
        }
    }
}
