package com.dzeio.openhealth.ui.weight

import android.Manifest
import android.content.SharedPreferences
import android.graphics.Paint
import android.os.Build
import android.os.Bundle
import android.view.LayoutInflater
import android.view.Menu
import android.view.MenuInflater
import android.view.MenuItem
import android.view.View
import android.view.ViewGroup
import android.widget.Toast
import androidx.activity.result.contract.ActivityResultContracts
import androidx.core.view.MenuProvider
import androidx.navigation.fragment.findNavController
import androidx.preference.PreferenceManager
import androidx.recyclerview.widget.LinearLayoutManager
import com.dzeio.charts.Entry
import com.dzeio.charts.axis.Line
import com.dzeio.charts.series.LineSerie
import com.dzeio.openhealth.BuildConfig
import com.dzeio.openhealth.R
import com.dzeio.openhealth.adapters.ItemAdapter
import com.dzeio.openhealth.core.BaseFragment
import com.dzeio.openhealth.data.weight.Weight
import com.dzeio.openhealth.databinding.FragmentListWeightBinding
import com.dzeio.openhealth.units.Mass
import com.dzeio.openhealth.utils.ChartUtils
import com.dzeio.openhealth.utils.PermissionsUtils
import com.google.android.material.color.MaterialColors
import dagger.hilt.android.AndroidEntryPoint
import java.text.DateFormat
import java.util.Date
import java.util.Locale
import kotlin.math.abs

@AndroidEntryPoint
class ListWeightFragment :
    BaseFragment<ListWeightViewModel, FragmentListWeightBinding>(ListWeightViewModel::class.java) {

    override val bindingInflater: (LayoutInflater, ViewGroup?, Boolean) ->
    FragmentListWeightBinding = FragmentListWeightBinding::inflate

    private lateinit var button: View
    private val activityResult = registerForActivityResult(
        ActivityResultContracts.RequestMultiplePermissions()
    ) {
        if (it.containsValue(false)) {
            // TODO: Show a popup with choice to change it
            Toast.makeText(requireContext(), R.string.permission_declined, Toast.LENGTH_LONG).show()
            return@registerForActivityResult
        }

        button.callOnClick()
    }

    private val menuProvider = object : MenuProvider {
        override fun onCreateMenu(menu: Menu, menuInflater: MenuInflater) {
            menu.findItem(R.id.action_add).isVisible = true
        }

        override fun onMenuItemSelected(item: MenuItem): Boolean {
            return when (item.itemId) {
                R.id.action_add -> {
                    findNavController().navigate(
                        ListWeightFragmentDirections.actionNavListWeightToNavWeightDialog(
                            WeightDialog.DialogTypes.ADD_WEIGHT.ordinal
                        )
                    )
                    true
                }

                else -> false
            }
        }
    }

    val settings: SharedPreferences by lazy {
        PreferenceManager.getDefaultSharedPreferences(requireContext())
    }

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)

        // Menu
        requireActivity().addMenuProvider(menuProvider)

        if (viewModel.goalWeight.value != null) {
            binding.goalButton.setText(R.string.edit_goal)
        }

        binding.goalButton.setOnClickListener {
            findNavController().navigate(
                ListWeightFragmentDirections.actionNavListWeightToNavWeightDialog(
                    WeightDialog.DialogTypes.EDIT_GOAL.ordinal
                )
            )
        }

        binding.bluetoothButton.setOnClickListener {
            val permissions = arrayListOf(
                Manifest.permission.BLUETOOTH,
                Manifest.permission.ACCESS_FINE_LOCATION
            )
            if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.S) {
                permissions.addAll(
                    arrayOf(
                        Manifest.permission.BLUETOOTH_CONNECT,
                        Manifest.permission.BLUETOOTH_SCAN
                    )
                )
            }
            val hasPermission = PermissionsUtils.hasPermission(requireContext(), permissions.toTypedArray())
            if (!hasPermission) {
                button = binding.bluetoothButton
                activityResult.launch(permissions.toTypedArray())
                return@setOnClickListener
            }
            findNavController().navigate(
                ListWeightFragmentDirections.actionNavListWeightToScanScalesDialog()
            )
        }

        val adapter = ItemAdapter<Weight>().apply {
            onItemClick = {
                findNavController().navigate(
                    ListWeightFragmentDirections.actionNavListWeightToNavEditWeight(
                        it.value.id
                    )
                )
            }
        }

        binding.list.apply {
            val manager = LinearLayoutManager(requireContext())
            layoutManager = manager
            this.adapter = adapter
        }

        viewModel.goalWeight.observe(viewLifecycleOwner) {
            val list = viewModel.weights.value
            if (list == null) {
                return@observe
            }
            updateGraph(list, it)
        }

        viewModel.massUnit.observe(viewLifecycleOwner) { unit ->
            binding.chart.yAxis.onValueFormat = { getString(unit.unit, unit.fromKilogram(it)) }
        }

        viewModel.weights.observe(viewLifecycleOwner) { list ->
            if (list != null) {
                val unit = viewModel.massUnit.value ?: Mass.KILOGRAM
                adapter.set(
                    list.map {
                        ItemAdapter.Item(
                            it,
                            getString(
                                unit.unit,
                                it.weight
                            ),
                            getString(
                                R.string.weight_item,
                                it.formatTimestamp(),
                                it.bmi ?: 0f,
                                it.totalBodyWater ?: 0f,
                                it.muscles ?: 0f,
                                it.leanBodyMass ?: 0f,
                                it.bodyFat ?: 0f,
                                it.boneMass ?: 0f,
                                it.visceralFat ?: 0f,
                                it.basalMetabolicRate ?: 0,
                                it.totalDailyEnergyExpendure ?: 0
                            ),
                            icon = R.drawable.ic_outline_edit_24
                        )
                    }.reversed()
                )
                updateGraph(list, viewModel.goalWeight.value)
            }
        }

        val chart = binding.chart

        chart.apply {
            series = arrayListOf(LineSerie(chart), LineSerie(chart))
            ChartUtils.materielTheme(this, requireView())
            (series[1] as LineSerie).linePaint.color = MaterialColors.getColor(
                requireView(),
                com.google.android.material.R.attr.colorSecondary
            )
            animator.enabled = false

            yAxis.apply {
                setYMin(null)
            }

            xAxis.apply {
                // 7 day history
                dataWidth = 6.048e+8 * 4
                scrollEnabled = true
                textPaint.textSize = 32f
                onValueFormat = onValueFormat@{
                    val formatter = DateFormat.getDateTimeInstance(
                        DateFormat.SHORT,
                        DateFormat.SHORT,
                        Locale.getDefault()
                    )
                    return@onValueFormat formatter.format(Date(it.toLong()))
                }
            }
        }

        // Debug button
        if (BuildConfig.DEBUG) {
            binding.debugRandomValues.visibility = View.VISIBLE
            binding.debugRandomValues.setOnClickListener {
                viewModel.generateRandomValues()
            }
            binding.debugRandomValues.setOnLongClickListener {
                viewModel.delete(viewModel.weights.value!!)
                return@setOnLongClickListener true
            }
        }
    }

    override fun onDestroyView() {
        super.onDestroyView()
        requireActivity().removeMenuProvider(menuProvider)
    }

    private fun updateGraph(list: List<Weight>, goal: Float? = null) {
        val chart = binding.chart
        val serie = chart.series[0] as LineSerie
        val avSerie = chart.series[1] as LineSerie

        val previous = 5
        val next = 5
        val averageList = arrayListOf<Entry>()
        for (i in list.indices) {
            val sub = list.subList((i - previous).coerceAtLeast(0), (i + next).coerceAtMost(list.size - 1))
            val average = (sub.map { it.weight }.reduceOrNull { acc, it -> it + acc } ?: 0f) / sub.size
            val it = list[i]
            averageList.add(
                Entry(
                    it.timestamp.toDouble(),
                    average
                )
            )
        }

        val entries: ArrayList<Entry> = list.map {
            Entry(
                it.timestamp.toDouble(),
                it.weight
            )
        } as ArrayList<Entry>

        val direction = averageList.last().y - averageList[averageList.size - 2].y

        chart.yAxis.clearLines()
        if (goal != null) {
            chart.yAxis.apply {
                addLine(
                    goal,
                    Line(true, Paint(linePaint).apply { strokeWidth = 4f })
                )
            }

            val dt = averageList.last().x - averageList[list.size - previous].x
            val timeUntilGoal = ((entries.last().y - goal) / abs(direction)).toLong() * dt

            binding.goalText.text = getString(
                R.string.days_until_goal_is_achieved,
                (timeUntilGoal / 1000 / 60 / 60 / 24).toInt()
            )
            entries.add(
                Entry(
                    (entries.last().x + timeUntilGoal),
                    goal
                )
            )
            binding.goalText.visibility = View.VISIBLE
        } else {
            entries.add(
                Entry(
                    entries.last().x + chart.xAxis.dataWidth!!,
                    entries.last().y + direction * 7 * 4
                )
            )
            binding.goalText.visibility = View.GONE
        }

        avSerie.entries = averageList
        serie.entries = entries

        if (list.isEmpty()) {
            chart.xAxis.x = 0.0
        } else {
            chart.xAxis.x = list.last().timestamp - chart.xAxis.dataWidth!!
        }

        chart.refresh()
    }
}
